/*
  Rhino test framework

   Copyright (C) 2011, 2012  Red Hat

This file is part of IcedTea.

IcedTea is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2, or (at your option)
any later version.

IcedTea is distributed in the hope that it will be useful, but
WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
General Public License for more details.

You should have received a copy of the GNU General Public License
along with IcedTea; see the file COPYING.  If not, write to the
Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
02110-1301 USA.

Linking this library statically or dynamically with other modules is
making a combined work based on this library.  Thus, the terms and
conditions of the GNU General Public License cover the whole
combination.

As a special exception, the copyright holders of this library give you
permission to link this library with independent modules to produce an
executable, regardless of the license terms of these independent
modules, and to copy and distribute the resulting executable under
terms of your choice, provided that you also meet, for each linked
independent module, the terms and conditions of the license of that
module.  An independent module is a module which is not derived from
or based on this library.  If you modify this library, you may extend
this exception to your version of the library, but you are not
obligated to do so.  If you do not wish to do so, delete this
exception statement from your version.
 */

package org.RhinoTests.Reporter;

import java.io.File;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.TreeMap;
import java.util.TreeSet;

/**
 * Main reporter class.
 *
 * @author Pavel Tisnovsky
 */
public class Reporter {

    private Map<String, Set<String>> logFiles = new TreeMap<String, Set<String>>();
    private Map<String, Map<String, List<String>>> testResults = new TreeMap<String, Map<String, List<String>>>();
    private Set<String> allTestNames = new TreeSet<String>();

    private CommandLineParameters params = null;

    public Reporter(String[] args) {
        this.params = new CommandLineParameters(args);
    }

    public void run() {
        readLogFiles();
        readTestResults();
        createListOfAllTests();
        IndexPageGenerator.generate(this.testResults, this.params);
        HistoryPagesGenerator.generate(this.testResults, this.params);
        GraphPagesGenerator.generate(this.testResults, this.params);
        LogPagesGenerator.generate(this.testResults, this.params);
    }

    private void readLogFiles() {
        File logDirectory = new File(this.params.getLogDir());
        System.out.println("logDir: " + logDirectory.getAbsolutePath());
        for (String subDir : logDirectory.list()) {
            File logSubDir = new File(logDirectory, subDir);
            Set<String> logFileSet = new TreeSet<String>();
            System.out.println("    subDir: " + logSubDir.getAbsolutePath());
            for (String logFileName : logSubDir.list()) {
                File logFile = new File(logSubDir, logFileName);
                System.out.println("        logFile: " + logFile.getPath());
                logFileSet.add(logFile.getAbsolutePath());
            }
            this.logFiles.put(subDir, logFileSet);
        }
        System.out.println(this.logFiles);
    }

    private void readTestResults() {
        for (Map.Entry<String, Set<String>> logFileEntry : this.logFiles.entrySet()) {
            String date = logFileEntry.getKey();
            Set<String> fileNames = logFileEntry.getValue();
            Map<String, List<String>> testResultsForOneDate = new TreeMap<String, List<String>>();
            for (String fileName : fileNames) {
                List<String> logFileContent = FileUtils.readTextFile(fileName);
                testResultsForOneDate.put(fileName, logFileContent);
            }
            this.testResults.put(date, testResultsForOneDate);
        }
    }

    /**
     * Fill up the set allTestNames which will contain names of all tests.
     * Because this set is implemented as TreeSet, the test names are
     * alphabetically sorted.
     */
    private void createListOfAllTests() {
        for (Map.Entry<String, Map<String, List<String>>> testResult: this.testResults.entrySet()) {
            for (Map.Entry<String, List<String>> testResultContent : testResult.getValue().entrySet()) {
                for (String oneLogLine : testResultContent.getValue()) {
                    if (oneLogLine.startsWith("PASSED: ")) {
                        String testName = oneLogLine.substring("PASSED: ".length());
                        this.allTestNames.add(testName);
                    }
                    if (oneLogLine.startsWith("FAILED: ")) {
                        String testName = oneLogLine.substring("FAILED: ".length());
                        testName = testName.substring(0, testName.indexOf(": FAILED")); 
                        this.allTestNames.add(testName);
                    }
                }
            }
        }
    }

    public static void main(String[] args) {
        new Reporter(args).run();
    }
}
